
#include "logging.h"

#include "odf_lists_styles_context.h"

#include "styles_list.h"
#include "style_text_properties.h"

#include <boost/regex.hpp>
#include <boost/lexical_cast.hpp>
 
namespace cpdoccore {

	using namespace odf_types;

namespace odf_writer {

odf_lists_styles_context::odf_lists_styles_context()
{
}
void odf_lists_styles_context::set_odf_context(odf_conversion_context * Context)
{
	odf_context_ = Context;
}

void odf_lists_styles_context::process_styles(office_element_ptr root )
{
	for (long i=0; i< lists_format_array_.size(); i++)
	{
		if (lists_format_array_[i].elements.size() < 1) continue;

		root->add_child_element(lists_format_array_[i].elements[0]);
	}
}	

void odf_lists_styles_context::add_style(int style_num, int based_num)
{
	//find index array for based_num

	std::wstring style_name;

	for (long i=0 ; i < lists_format_array_.size(); i++)
	{
		if (lists_format_array_[i].oox_based_number == based_num)
		{
			link_format_map_[style_num]  = lists_format_array_[i].odf_list_style_name;
			break;
		}
	}
}
std::wstring odf_lists_styles_context::get_style_name1(int oox_style_num)
{
	for (long i=0 ; i < lists_format_array_.size(); i++)
	{
		if (lists_format_array_[i].oox_based_number == oox_style_num)
		{
			return lists_format_array_[i].odf_list_style_name;
		}
	}
	return L"";
}
std::wstring odf_lists_styles_context::get_style_name(int oox_style_num)
{
    if (link_format_map_.count(oox_style_num) > 0)
	{
        return link_format_map_.at(oox_style_num);
	}
	else
	{
		return L"";
	}
}
void odf_lists_styles_context::start_style(int based_number)
{
	list_format_state state;

	office_element_ptr elm;	
	create_element(L"text", L"list-style", elm, odf_context_);
	state.elements.push_back(elm);
	
	state.oox_based_number = based_number;
	state.odf_list_style_name = std::wstring(L"WWNum") + boost::lexical_cast<std::wstring>(based_number + 1);

	text_list_style *style = dynamic_cast<text_list_style *>(elm.get());
	if (style == NULL)return;

	style->text_list_style_attr_.style_name_ = state.odf_list_style_name;

	lists_format_array_.push_back(state); //перенести в end??
}

style_list_level_properties * odf_lists_styles_context::get_list_level_properties()
{
	if (lists_format_array_.size() < 1) return NULL;
	if (lists_format_array_.back().elements.size() <1) return NULL;
	
	text_list_level_style_number *style_number_ = dynamic_cast<text_list_level_style_number *>(lists_format_array_.back().elements.back().get());
	text_list_level_style_bullet *style_bullet_ = dynamic_cast<text_list_level_style_bullet *>(lists_format_array_.back().elements.back().get());
	text_list_level_style_image  *style_image_  = dynamic_cast<text_list_level_style_image  *>(lists_format_array_.back().elements.back().get());

	style_list_level_properties *props = NULL;
	if (style_number_)
	{
 		if (!style_number_->style_list_level_properties_)
			create_element(L"style", L"list-level-properties",style_number_->style_list_level_properties_,odf_context_);

		props = dynamic_cast<style_list_level_properties *>(style_number_->style_list_level_properties_.get());    
	}
	if (style_bullet_)
	{
 		if (!style_bullet_->style_list_level_properties_)
			create_element(L"style", L"list-level-properties",style_bullet_->style_list_level_properties_,odf_context_);

		props =  dynamic_cast<style_list_level_properties *>(style_bullet_->style_list_level_properties_.get());    
	}
	if (style_image_)
	{
 		if (!style_image_->style_list_level_properties_)
			create_element(L"style", L"list-level-properties",style_image_->style_list_level_properties_,odf_context_);

		props =  dynamic_cast<style_list_level_properties *>(style_image_->style_list_level_properties_.get());    
	}

	return props;
}
style_text_properties * odf_lists_styles_context::get_text_properties()
{
	if (lists_format_array_.size() < 1) return NULL;
	if (lists_format_array_.back().elements.size() <1) return NULL;
	
	text_list_level_style_number *style_number_ = dynamic_cast<text_list_level_style_number *>(lists_format_array_.back().elements.back().get());
	text_list_level_style_bullet *style_bullet_ = dynamic_cast<text_list_level_style_bullet *>(lists_format_array_.back().elements.back().get());
	text_list_level_style_image  *style_image_  = dynamic_cast<text_list_level_style_image  *>(lists_format_array_.back().elements.back().get());

	style_text_properties *props = NULL;
	if (style_number_)
	{
 		if (!style_number_->style_text_properties_)
			create_element(L"style", L"text-properties",style_number_->style_text_properties_,odf_context_);

		props = dynamic_cast<style_text_properties *>(style_number_->style_text_properties_.get());    
	}
	if (style_bullet_)
	{
 		if (!style_bullet_->style_text_properties_)
			create_element(L"style", L"text-properties",style_bullet_->style_text_properties_,odf_context_);

		props =  dynamic_cast<style_text_properties *>(style_bullet_->style_text_properties_.get());    
	}
	if (style_image_)
	{
 		if (!style_image_->style_list_level_properties_)
			create_element(L"style", L"list-level-properties",style_image_->style_list_level_properties_,odf_context_);

		props =  dynamic_cast<style_text_properties *>(style_image_->style_text_properties_.get());    
	}
	return props;
}

style_list_level_label_alignment * odf_lists_styles_context::get_list_level_alignment_properties()
{
	if (lists_format_array_.size() < 1) return NULL;
	if (lists_format_array_.back().elements.size() <1) return NULL;
	
	text_list_level_style_number *style_number_ = dynamic_cast<text_list_level_style_number *>(lists_format_array_.back().elements.back().get());
	text_list_level_style_bullet *style_bullet_ = dynamic_cast<text_list_level_style_bullet *>(lists_format_array_.back().elements.back().get());
	text_list_level_style_image  *style_image_  = dynamic_cast<text_list_level_style_image  *>(lists_format_array_.back().elements.back().get());

	style_list_level_properties *props = NULL;
	if (style_number_)
	{
 		if (!style_number_->style_list_level_properties_)
			create_element(L"style", L"list-level-properties",style_number_->style_list_level_properties_,odf_context_);

		props = dynamic_cast<style_list_level_properties *>(style_number_->style_list_level_properties_.get());    
	}
	if (style_bullet_)
	{
 		if (!style_bullet_->style_list_level_properties_)
			create_element(L"style", L"list-level-properties",style_bullet_->style_list_level_properties_,odf_context_);

		props =  dynamic_cast<style_list_level_properties *>(style_bullet_->style_list_level_properties_.get());    
	}
	if (style_image_)
	{
 		if (!style_image_->style_list_level_properties_)
			create_element(L"style", L"list-level-properties",style_image_->style_list_level_properties_,odf_context_);

		props =  dynamic_cast<style_list_level_properties *>(style_image_->style_list_level_properties_.get());    
	}
	if (!props)return NULL;

	if (!props->style_list_level_label_alignment_)
	{
		create_element(L"style", L"list-level-label-alignment" ,props->style_list_level_label_alignment_,odf_context_);
	}
	return dynamic_cast<style_list_level_label_alignment *>(props->style_list_level_label_alignment_.get());
}

int odf_lists_styles_context::start_style_level(int level, int type)
{
	if (lists_format_array_.size() < 1) return -1;
	int odf_type =1;
	int format_type = -1;

	std::wstring num_format = L"1"; 
	bool sync_letter = false;
	switch(type)
	{
		case 0: //numberformatAiueo                        : 
		case 1: //numberformatAiueoFullWidth               :
			num_format = L"ア, イ, ウ, ...";
			break;
		case 2: //numberformatArabicAbjad                  : 
			break;
		case 3: //numberformatArabicAlpha                  : 
			break;
		case 4: //numberformatBahtText                     : 
			break;
		case 5: //numberformatBullet                       :
			odf_type = 2;
			break;
		case 6: //numberformatCardinalText                 : 
			break;
		case 7: //numberformatChicago                      : 
			break;
		case 8: //numberformatChineseCounting              : 
			num_format = L"ｲ, ﾛ, ﾊ, ...";
			break;
		case 9: //numberformatChineseCountingThousand      : 
			break;
		case 10: //numberformatChineseLegalSimplified       :
			num_format = L"一, 二, 三, ...";
			break;
		case 11: //numberformatChosung                      : 
			break;
		case 12: //numberformatCustom                       : 
			break;
		case 13: //numberformatDecimal                      : 
			num_format = L"1";
			break;
		case 14: //numberformatDecimalEnclosedCircle        : 
			break;
		case 15: //numberformatDecimalEnclosedCircleChinese : 
			break;
		case 16: //numberformatDecimalEnclosedFullstop      : 
			break;
		case 17: //numberformatDecimalEnclosedParen         : 
			break;
		case 18: //numberformatDecimalFullWidth             : 
			break;
		case 19: //numberformatDecimalFullWidth2            : 
			break;
		case 20: //numberformatDecimalHalfWidth             : 
			break;
		case 21: //numberformatDecimalZero                  : 
			//01 02 03 ...
			break;
		case 22: //numberformatDollarText                   : 
			break;
		case 23: //numberformatGanada                       : 
			break;
		case 24: //numberformatHebrew1                      : 
			break;
		case 25: //numberformatHebrew2                      : 
			break;
		case 26: //numberformatHex                          : 
			break;
		case 27: //numberformatHindiConsonants              : 
			break;
		case 28: //numberformatHindiCounting                : 
			break;
		case 29: //numberformatHindiNumbers                 : 
			break;
		case 30: //numberformatHindiVowels                  :
			break;
		case 31: //numberformatIdeographDigital             : 
			break;
		case 32: //numberformatIdeographEnclosedCircle      : 
			break;
		case 33: //numberformatIdeographLegalTraditional    : 
			num_format = L"壹, 貳, 參, ...";
			break;
		case 34: //numberformatIdeographTraditional         :
			num_format = L"甲, 乙, 丙, ...";
			break;
		case 35: //numberformatIdeographZodiac              : 
			num_format = L"子, 丑, 寅, ...";
			break;
		case 36: //numberformatIdeographZodiacTraditional   : 
			num_format = L"甲子, 乙丑, 丙寅, ...";
			break;
		case 37: //numberformatIroha                        : 
			num_format = L"ｲ, ﾛ, ﾊ, ...";
			break;
		case 38: //numberformatIrohaFullWidth               : 
			break;
		case 39: //numberformatJapaneseCounting             : 
			break;
		case 40: //numberformatJapaneseDigitalTenThousand   : 
			break;
		case 41: //numberformatJapaneseLegal                : 
			break;
		case 42: //numberformatKoreanCounting               : 
		case 43: //numberformatKoreanDigital                : 
			num_format = L"일, 이, 삼, ...";
			break;
		case 44: //numberformatKoreanDigital2               : 
			break;
		case 45: //numberformatKoreanLegal                  : 
			break;
		case 46: //numberformatLowerLetter 
			num_format = L"a";
			sync_letter = true; 
			break;
		case 47: //numberformatLowerRoman                   : 
			num_format = L"i";
			break;
		case 48: //numberformatNone                         : 
			num_format = L"";
			break;
		case 49: //numberformatNumberInDash                 : //??
			//suffix -
			//preffix -
			break;
		case 50: //numberformatOrdinal                      : 
			break;
		case 51: //numberformatOrdinalText                  : 
			break;
		case 52: //numberformatRussianLower                 : 
			num_format = L"А, Б, .., Аа, Аб, ... (ru)";
			sync_letter = true; 
			break;
		case 53: //numberformatRussianUpper                 : 
			num_format = L"А, Б, .., Аа, Аб, ... (ru)";
			sync_letter = true; 
			break;
		case 54: //numberformatTaiwaneseCounting            : 
			break;
		case 55: //numberformatTaiwaneseCountingThousand    : 
			break;
		case 56: //numberformatTaiwaneseDigital             : 
			break;
		case 57: //numberformatThaiCounting                 : 
			break;
		case 58: //numberformatThaiLetters                  : 
			break;
		case 59: //numberformatThaiNumbers                  : 
			break;
		case 60: //numberformatUpperLetter                  : 
			num_format = L"A";
			sync_letter = true; 
			break;
		case 61: //numberformatUpperRoman                   : 
			num_format = L"I";
			break;
		case 62: //numberformatVietnameseCounting           : 
			break;
		case 1000:
			odf_type = 3;
			break;
		default :
			odf_type = 1;
			break;
	}

	office_element_ptr elm;	
	
	if (odf_type == 2)
		create_element(L"text", L"list-level-style-bullet", elm, odf_context_);
	else if (odf_type == 3)
		create_element(L"text", L"list-level-style-image", elm, odf_context_);
	else
		create_element(L"text", L"list-level-style-number", elm, odf_context_);

	text_list_level_style_number *style_number_ = dynamic_cast<text_list_level_style_number *>(elm.get());
	text_list_level_style_bullet *style_bullet_ = dynamic_cast<text_list_level_style_bullet *>(elm.get());
	text_list_level_style_image  *style_image_  = dynamic_cast<text_list_level_style_image  *>(elm.get());

	if (style_number_)
	{
		style_number_->text_list_level_style_attr_.text_level_ = level + 1;
		style_number_->text_list_level_style_number_attr_.common_num_format_attlist_.style_num_format_ = num_format;

		if (sync_letter)style_number_->text_list_level_style_number_attr_.common_num_format_attlist_.style_num_letter_sync_ = true;
	}

	if (style_image_)
	{
		style_image_->text_list_level_style_attr_.text_level_ = level + 1;
	}
	if (style_bullet_)
	{
		style_bullet_->text_list_level_style_attr_.text_level_ = level + 1;
	}
	lists_format_array_.back().elements.push_back(elm);

	lists_format_array_.back().elements[0]->add_child_element(elm);
	
	return odf_type;
}
wchar_t convert_bullet_char(wchar_t c)
{
    wchar_t res = c;
    switch(c)
    {
    case L'\xf075': res = L'\xE00C';; break; 
    case L'\xf0A7': res = L'\xE00A'; break; 
    case L'\xf0FE': res = L'\x2611'; break; 
    case L'\xf0B7': res = L'\x25CF'; break; 
    case L'\xf0D8': res = L'\x27A2'; break; 
    case L'\xf0FC': res = L'\x2714'; break; 
    case L'\xf06F': res = L'\x25CB'; break; 
    case L'\xf0DA': res = L'\x2794'; break; 
    case L'\xf04F': res = L'\x2717'; break; 
    default:break;
    }

    return res;
}
void odf_lists_styles_context::set_numeric_format(std::wstring val)
{
	if (lists_format_array_.size() < 1) return;
	if (lists_format_array_.back().elements.size() <1) return ;
	if ( val.length() <1 ) 
		return;

	text_list_level_style_number *style_number_ = dynamic_cast<text_list_level_style_number *>(lists_format_array_.back().elements.back().get());
	if (style_number_)
	{
		//style_number_->text_list_level_style_number_attr_.common_num_format_attlist_.style_num_format_ = val;

		std::wstring s = val;

		int r1 = s.find(L"%");
		if (r1 >0)
		{
			std::wstring prefix = s.substr(0, r1);
			style_number_->text_list_level_style_number_attr_.common_num_format_prefix_suffix_attlist_.style_num_prefix_ = prefix;

			s = s.substr(r1, s.length() - r1);
			r1 = 0;
		}
		if (r1 < 0) return;//??

		r1 = s.find(L"%", 1);
		if (r1 >0)
		{
			int level = 0;
			//find count % in s
			int r2=0;
			while (r2 < s.length())
			{
				r2 = s.find(L"%", r2);
				if (r2 >=0) level++;
				else break;
				r2++;
			}
			style_number_->text_list_level_style_number_attr_.text_display_levels_ = level;

			s = s.substr(0,r1);
		}
		
		boost::wregex r(L"(\\%[0-9]+)(\\S+)");
		boost::match_results<std::wstring::const_iterator> res;
		if ( boost::regex_match(s, res, r))
		{
			std::wstring suffix = res[2].str();
			if (!suffix.empty()) 
				style_number_->text_list_level_style_number_attr_.common_num_format_prefix_suffix_attlist_.style_num_suffix_ = suffix;
		
			std::wstring numb = res[2].str();
			//if (!numb.empty())
			//{
			//	int format = 0;
			//	try
			//	{
			//		format = boost::lexical_cast<int>(numb.substr(1, numb.length() - 1));
			//	}catch(...)
			//	{
			//	}
			//	if (format > 0)
			//	{
			//		//style_number_->text_list_level_style_number_attr_.common_num_format_attlist_.text_display_levels_ = display_level;
			//	}
			//}

		}

	}
}
void odf_lists_styles_context::set_bullet_char(std::wstring val)
{
	if (lists_format_array_.size() < 1) return;
	if (lists_format_array_.back().elements.size() <1) return ;
	if ( val.length() <1 ) 
		return;
	
	text_list_level_style_bullet *style_bullet_ = dynamic_cast<text_list_level_style_bullet *>(lists_format_array_.back().elements.back().get());

	if (style_bullet_ == NULL) return;
	wchar_t char_ = convert_bullet_char(val.at(0));
	style_bullet_->text_list_level_style_bullet_attr_.text_bullet_char_ = char_;
}
void odf_lists_styles_context::set_bullet_image_size(double size)
{
	if (size < 0.1) return;
	if (lists_format_array_.size() < 1) return;
	if (lists_format_array_.back().elements.size() <1) return ;

	style_list_level_properties *props = get_list_level_properties();

	if(!props) return;
	
	props->fo_width_  = length(length(size,length::pt).get_value_unit(length::cm),length::cm);
	props->fo_height_ = length(length(size,length::pt).get_value_unit(length::cm),length::cm);
}
void odf_lists_styles_context::set_bullet_image	(std::wstring ref)
{
	if (lists_format_array_.size() < 1) return;
	if (lists_format_array_.back().elements.size() <1) return ;
	
	text_list_level_style_image *style_image_ = dynamic_cast<text_list_level_style_image *>(lists_format_array_.back().elements.back().get());

	if (style_image_ == NULL) return;

	style_image_->text_list_level_style_image_attr_.common_xlink_attlist_.href_ = ref;
	
	style_image_->text_list_level_style_image_attr_.common_xlink_attlist_.type_= xlink_type::Simple;
	style_image_->text_list_level_style_image_attr_.common_xlink_attlist_.show_ = xlink_show::Embed;
	style_image_->text_list_level_style_image_attr_.common_xlink_attlist_.actuate_= xlink_actuate::OnLoad;
}
void odf_lists_styles_context::set_start_number(int val)
{
	if (lists_format_array_.size() < 1) return;
	if (lists_format_array_.back().elements.size() <1) return ;

	text_list_level_style_number *style_number_ = dynamic_cast<text_list_level_style_number *>(lists_format_array_.back().elements.back().get());

	if (style_number_ == NULL) return;

	style_number_->text_list_level_style_number_attr_.text_start_value_ = val;
}
void odf_lists_styles_context::set_text_style_name(std::wstring name)
{
	if (lists_format_array_.size() < 1) return;
	if (lists_format_array_.back().elements.size() <1) return ;
	
	text_list_level_style_number *style_number_ = dynamic_cast<text_list_level_style_number *>(lists_format_array_.back().elements.back().get());
	text_list_level_style_bullet *style_bullet_ = dynamic_cast<text_list_level_style_bullet *>(lists_format_array_.back().elements.back().get());

	if (style_number_) 	style_number_->text_list_level_style_number_attr_.text_style_name_ = name;
	if (style_bullet_) 	style_bullet_->text_list_level_style_bullet_attr_.text_style_name_ = name;

}
void odf_lists_styles_context::end_style_level()
{
}
void odf_lists_styles_context::end_style()
{
}
}
}
